@@ -1,19 +1,18 @@
 /* Conversion of files between different charsets and surfaces.
-   Copyright  1990, 92, 93, 94, 96, 97, 98, 99 Free Software Foundation, Inc.
-   This file is part of the GNU C Library.
+   Copyright  1990,92,93,94,96,97,98,99,00 Free Software Foundation, Inc.
    Contributed by Franois Pinard <pinard@iro.umontreal.ca>, 1990.
 
-   The `recode' Library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public License
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.
 
-   The `recode' Library is distributed in the hope that it will be
+   This library is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with the `recode' Library; see the file `COPYING.LIB'.
    If not, write to the Free Software Foundation, Inc., 59 Temple Place -
    Suite 330, Boston, MA 02111-1307, USA.  */
@@ -23,58 +22,27 @@
 
 /* FIXME: Cleanup memory at end of job, and softly report errors.  */
 
-/* Date: 1998-12-11 20:20 -0500
-X-From-Line: recode-forum-owner@IRO.UMontreal.CA  Fri Dec 11 21:22:51 1998
-Return-Path: <recode-forum-owner@iro.umontreal.ca>
-Received: from degusse.IRO.UMontreal.CA (degusse.IRO.UMontreal.CA [132.204.24.51])
-	by lagrande.IRO.UMontreal.CA (8.9.1/8.9.1) with ESMTP id VAA08222
-	for <pinard@lagrande.IRO.UMontreal.CA>; Fri, 11 Dec 1998 21:22:51 -0500 (EST)
-Received: by degusse.IRO.UMontreal.CA (8.9.1/8.9.1) id VAA11542
-	for recode-forum-outgoing; Fri, 11 Dec 1998 21:22:44 -0500 (EST)
-Received: from jupiter.rtsq.qc.ca (rtsq.grics.qc.ca [199.84.132.81])
-	by degusse.IRO.UMontreal.CA (8.9.1/8.9.1) with ESMTP id VAA11529
-	for <recode-forum@iro.umontreal.ca>; Fri, 11 Dec 1998 21:22:38 -0500 (EST)
-Received: from ariel.progiciels-bpi.ca by jupiter.rtsq.qc.ca (8.8.8/8.8.8) with SMTP id VAA10935 for <@jupiter.rtsq.qc.ca:recode-forum@iro.umontreal.ca>; Fri, 11 Dec 1998 21:22:10 -0500
-Received: from icule.progiciels-bpi.ca (uucp@localhost) by ariel.progiciels-bpi.ca (950413.SGI.8.6.12/950213.SGI) via UUCP id VAA17236 for recode-forum@iro.umontreal.ca; Fri, 11 Dec 1998 21:23:51 -0800
-Received: by icule.progiciels-bpi.ca (8.8.8/8.8.8) id UAA05839; Fri, 11 Dec 1998 20:21:00 -0500
-X-Face: "b_m|CE6#'Q8fliQrwHl9K,]PA_o'*S~Dva{~b1n*)K*A(BIwQW.:LY?t4~xhYka_.LV?Qq
- `}X|71X0ea&H]9Dsk!`kxBXlG;q$mLfv_vtaHK_rHFKu]4'<*LWCyUe@ZcI6"*wB5M@[m<Ok5/cC^=
- CxDhg=TJi^o[E
-Message-ID: <oq90ge17gk.fsf@icule.progiciels-bpi.ca>
-Mime-Version: 1.0
-Content-Type: text/plain; charset=iso-8859-1
-Content-Transfer-Encoding: 8bit
-Sender: owner-recode-forum@IRO.UMontreal.CA
-Precedence: bulk
-Lines: 49
-Xref: titan.progiciels-bpi.ca entretien.recode:1739
-
-   Let me take a few minutes of rest, and share with you the state of combining
-   in `recode'.  Fun enough, my initial writing of it, which was using some
-   space while it was boiling, reduced to something small and rather simple
-   after some time, from merging of structures and functions, and removal of
-   some limits.  It does not even show, now, that it once was more complicated!
-
-   The satisfactory aspects are that `recode' is now able to combine a set of
+/* The satisfactory aspects are that `recode' is now able to combine a set of
    sequence of UCS-2 characters into single codes, or explode those single
-   codes into the original sequence.  It may happen that many sequences
-   reduce to the same code, one of them is arbitrarily taken as canonical.
-   Any combining sequence of an equivalent set produces the equivalent code,
-   but at explode time, codes are always turned into their canonical sequence.
-   I did not limit the length of a combining sequence, yet it is usually small.
-   Also, I did not put any limit on the number of possibly equivalent sequences.
-   For combining, for each possible partial match in any sequence, there is
-   a state.  States corresponding to the scan of a single code are found by
-   hash coding, while other states are linearly searched on a linked list of
-   possible shifts, rooted on the state corresponding to the partial sequence,
-   but without its last code.  I expect this combination of methods to execute
-   reasonably fast.  Exploding is much simpler, and I merely use hash coding.
+   codes into the original sequence.  It may happen that many sequences reduce
+   to the same code, one of them is arbitrarily taken as canonical.  Any
+   combining sequence of an equivalent set produces the equivalent code, but
+   at explode time, codes are always turned into their canonical sequence.  I
+   did not limit the length of a combining sequence, yet it is usually small.
+   Also, I did not put any limit on the number of possibly equivalent
+   sequences.  For combining, for each possible partial match in any sequence,
+   there is a state.  States corresponding to the scan of a single code are
+   found by hash coding, while other states are linearly searched on a linked
+   list of possible shifts, rooted on the state corresponding to the partial
+   sequence, but without its last code.  I expect this combination of methods
+   to execute reasonably fast.  Exploding is much simpler, and I merely use
+   hash coding.
 
-   The less satisfactory aspects are that the user interface is still
-   very rude.  For the time being, I merely added a "combined" charset:
-   combining is done by `u2..co', exploding is done by `co..u2'.  We surely
-   could do much better, and build on the concept of options (introduced by
-   `+' in the request syntax), Options are probably going to be used for other
+   The less satisfactory aspects are that the user interface is still very
+   rude.  For the time being, I merely added a "combined" charset: combining
+   is done by `u2..co', exploding is done by `co..u2'.  We surely could do
+   much better, and build on the concept of options (introduced by `+' in the
+   request syntax), Options are probably going to be used for other
    transliteration matters.  Even if built on UCS-2, combining matters would
    have natural extensions in other _smaller_ charsets, and it would be nice
    being able to derive the functionality automatically.  One difficulty to
@@ -83,15 +51,10 @@ Xref: titan.progiciels-bpi.ca entretien.recode:1739
    UCS-2 is not otherwise mandated in the request, we would have to create
    such an UCS-2 intermediate.  On the other hand, if UCS-2 is sandwidched
    somewhere in a complex series of steps, we should implicitly propagate
-   options towards the UCS-2 step.  Even if nothing looks utterly difficult,
-   I guess that for doing it right, a lot of attention and care is needed.
+   options towards the UCS-2 step.  Even if nothing looks utterly difficult, I
+   guess that for doing it right, a lot of attention and care is needed.
    Also, I'm pretty sure that scrutinizing the documents (from w3.org or
-   Unicode) that were suggested to us will raise new specific concerns.
-
-   In any case, I'll try to clean a bit more around, for a few hours or maybe
-   a few days, who knows :-), and prepare a new pretest offering "combined".
-   Hmph!  Some documentation, even if terse, would also be welcome.  More
-   later.  */
+   Unicode) that were suggested to us will raise new specific concerns.  */
 
 /* Exploding.  */
 
@@ -156,12 +119,12 @@ init_explode (RECODE_STEP step,
 `------------------------------------*/
 
 bool
-explode_byte_byte (RECODE_CONST_STEP step, RECODE_TASK task)
+explode_byte_byte (RECODE_SUBTASK subtask)
 {
-  Hash_table *table = step->step_table;
+  Hash_table *table = subtask->step->step_table;
   unsigned value;
 
-  while (value = get_byte (task), value != EOF)
+  while (value = get_byte (subtask), value != EOF)
     {
       unsigned short lookup = value;
       unsigned short *result = hash_lookup (table, &lookup);
@@ -171,24 +134,24 @@ explode_byte_byte (RECODE_CONST_STEP step, RECODE_TASK task)
 	  result++;
 	  while (*result != DONE && *result != ELSE)
 	    {
-	      put_byte (*result, task);
+	      put_byte (*result, subtask);
 	      result++;
 	    }
 	}
       else
-	put_byte (value, task);
+	put_byte (value, subtask);
     }
 
-  TASK_RETURN (task);
+  SUBTASK_RETURN (subtask);
 }
 
 bool
-explode_ucs2_byte (RECODE_CONST_STEP step, RECODE_TASK task)
+explode_ucs2_byte (RECODE_SUBTASK subtask)
 {
-  Hash_table *table = step->step_table;
+  Hash_table *table = subtask->step->step_table;
   unsigned value;
 
-  while (get_ucs2 (&value, step, task))
+  while (get_ucs2 (&value, subtask))
     {
       unsigned short lookup = value;
       unsigned short *result = hash_lookup (table, &lookup);
@@ -198,27 +161,27 @@ explode_ucs2_byte (RECODE_CONST_STEP step, RECODE_TASK task)
 	  result++;
 	  while (*result != DONE && *result != ELSE)
 	    {
-	      put_byte (*result, task);
+	      put_byte (*result, subtask);
 	      result++;
 	    }
 	}
       else
-	put_byte (value, task);
+	put_byte (value, subtask);
     }
 
-  TASK_RETURN (task);
+  SUBTASK_RETURN (subtask);
 }
 
 bool
-explode_byte_ucs2 (RECODE_CONST_STEP step, RECODE_TASK task)
+explode_byte_ucs2 (RECODE_SUBTASK subtask)
 {
-  Hash_table *table = step->step_table;
+  Hash_table *table = subtask->step->step_table;
   unsigned value;
 
-  if (value = get_byte (task), value != EOF)
+  if (value = get_byte (subtask), value != EOF)
     {
-      if (task->byte_order_mark)
-	put_ucs2 (BYTE_ORDER_MARK, task);
+      if (subtask->task->byte_order_mark)
+	put_ucs2 (BYTE_ORDER_MARK, subtask);
 
       while (true)
 	{
@@ -229,29 +192,29 @@ explode_byte_ucs2 (RECODE_CONST_STEP step, RECODE_TASK task)
 	    {
 	      result++;
 	      while (*result != DONE && *result != ELSE)
-		put_ucs2 (*result++, task);
+		put_ucs2 (*result++, subtask);
 	    }
 	  else
-	    put_ucs2 (value, task);
+	    put_ucs2 (value, subtask);
 
-	  if (value = get_byte (task), value == EOF)
+	  if (value = get_byte (subtask), value == EOF)
 	    break;
 	}
     }
 
-  TASK_RETURN (task);
+  SUBTASK_RETURN (subtask);
 }
 
 bool
-explode_ucs2_ucs2 (RECODE_CONST_STEP step, RECODE_TASK task)
+explode_ucs2_ucs2 (RECODE_SUBTASK subtask)
 {
-  Hash_table *table = step->step_table;
+  Hash_table *table = subtask->step->step_table;
   unsigned value;
 
-  if (get_ucs2 (&value, step, task))
+  if (get_ucs2 (&value, subtask))
     {
-      if (task->byte_order_mark)
-	put_ucs2 (BYTE_ORDER_MARK, task);
+      if (subtask->task->byte_order_mark)
+	put_ucs2 (BYTE_ORDER_MARK, subtask);
 
       while (true)
 	{
@@ -262,17 +225,17 @@ explode_ucs2_ucs2 (RECODE_CONST_STEP step, RECODE_TASK task)
 	    {
 	      result++;
 	      while (*result != DONE && *result != ELSE)
-		put_ucs2 (*result++, task);
+		put_ucs2 (*result++, subtask);
 	    }
 	  else
-	    put_ucs2 (value, task);
+	    put_ucs2 (value, subtask);
 
-	  if (!get_ucs2 (&value, step, task))
+	  if (!get_ucs2 (&value, subtask))
 	    break;
 	}
     }
 
-  TASK_RETURN (task);
+  SUBTASK_RETURN (subtask);
 }
 
 /* Combining.  */
@@ -292,7 +255,7 @@ struct state
   unsigned short result;	/* character equivalent to the combination */
   struct state *shift;		/* list of states for one more character */
   struct state *unshift;	/* state for one less character (back link) */
-  struct state *next;		/* next state within a linked chain of states */
+  struct state *next;		/* next state in a linked chain of states */
 };
 
 /*---------------------------.
@@ -316,8 +279,23 @@ state_compare (const void *void_first, const void *void_second)
   return first->character == second->character;
 }
 
+static void
+state_free (void *void_state)
+{
+  struct state *state = void_state;
+  struct state *shift = state->shift;
+
+  while (shift != NULL)
+    {
+      struct state *next_shift = shift->next;
+      free (shift);
+      shift = next_shift;
+    }
+  free (state);
+}
+
 static struct state *
-prepare_shifted_state (struct state *state, unsigned short character,
+prepare_shifted_state (struct state *state, unsigned character,
 		       RECODE_CONST_STEP step)
 {
   if (state)
@@ -367,7 +345,7 @@ prepare_shifted_state (struct state *state, unsigned short character,
 }
 
 static struct state *
-find_shifted_state (struct state *state, unsigned short character,
+find_shifted_state (struct state *state, unsigned character,
 		    RECODE_CONST_STEP step)
 {
   if (state)
@@ -404,7 +382,7 @@ init_combine (RECODE_STEP step,
   if (before_options || after_options)
     return false;
 
-  table = hash_initialize (0, NULL, state_hash, state_compare, NULL);
+  table = hash_initialize (0, NULL, state_hash, state_compare, state_free);
 
   if (!table)
     return false;
@@ -465,27 +443,27 @@ init_combine (RECODE_STEP step,
    If we later find that it does, backtracing will have to be revisited.  */
 
 static void
-backtrack_byte (struct state *state,  RECODE_TASK task)
+backtrack_byte (struct state *state, RECODE_SUBTASK subtask)
 {
   if (state->result == NOT_A_CHARACTER)
     {
-      backtrack_byte (state->unshift, task);
-      put_byte (state->character, task);
+      backtrack_byte (state->unshift, subtask);
+      put_byte (state->character, subtask);
     }
   else
-    put_byte (state->result, task);
+    put_byte (state->result, subtask);
 }
 
 static void
-backtrack_ucs2 (struct state *state,  RECODE_TASK task)
+backtrack_ucs2 (struct state *state,  RECODE_SUBTASK subtask)
 {
   if (state->result == NOT_A_CHARACTER)
     {
-      backtrack_ucs2 (state->unshift, task);
-      put_ucs2 (state->character, task);
+      backtrack_ucs2 (state->unshift, subtask);
+      put_ucs2 (state->character, subtask);
     }
   else
-    put_ucs2 (state->result, task);
+    put_ucs2 (state->result, subtask);
 }
 
 /*------------------------------------.
@@ -493,185 +471,197 @@ backtrack_ucs2 (struct state *state,  RECODE_TASK task)
 `------------------------------------*/
 
 bool
-combine_byte_byte (RECODE_CONST_STEP step, RECODE_TASK task)
+combine_byte_byte (RECODE_SUBTASK subtask)
 {
   struct state *state = NULL;
   unsigned value;
 
-  if (value = get_byte (task), value != EOF)
+  if (value = get_byte (subtask), value != EOF)
     {
       while (true)
 	{
-	  struct state *shift = find_shifted_state (state, value, step);
+	  struct state *shift
+	    = find_shifted_state (state, value, subtask->step);
 
 	  if (shift)
 	    {
 	      state = shift;
-	      if (value = get_byte (task), value == EOF)
+	      if (value = get_byte (subtask), value == EOF)
 		break;
 	    }
 	  else if (state)
 	    {
 	      if (state->result == NOT_A_CHARACTER)
-		backtrack_byte (state, task);
+		backtrack_byte (state, subtask);
 	      else
-		put_byte (state->result, task);
+		put_byte (state->result, subtask);
 	      state = NULL;
 	    }
 	  else
 	    {
-	      put_byte (value, task);
-	      if (value = get_byte (task), value == EOF)
+	      put_byte (value, subtask);
+	      if (value = get_byte (subtask), value == EOF)
 		break;
 	    }
 	}
 
       if (state)
-	if (state->result == NOT_A_CHARACTER)
-	  backtrack_byte (state, task);
-	else
-	  put_byte (state->result, task);
+	{
+	  if (state->result == NOT_A_CHARACTER)
+	    backtrack_byte (state, subtask);
+	  else
+	    put_byte (state->result, subtask);
+	}
     }
 
-  TASK_RETURN (task);
+  SUBTASK_RETURN (subtask);
 }
 
 bool
-combine_ucs2_byte (RECODE_CONST_STEP step, RECODE_TASK task)
+combine_ucs2_byte (RECODE_SUBTASK subtask)
 {
   struct state *state = NULL;
   unsigned value;
 
-  if (get_ucs2 (&value, step, task))
+  if (get_ucs2 (&value, subtask))
     {
       while (true)
 	{
-	  struct state *shift = find_shifted_state (state, value, step);
+	  struct state *shift
+	    = find_shifted_state (state, value, subtask->step);
 
 	  if (shift)
 	    {
 	      state = shift;
-	      if (!get_ucs2 (&value, step, task))
+	      if (!get_ucs2 (&value, subtask))
 		break;
 	    }
 	  else if (state)
 	    {
 	      if (state->result == NOT_A_CHARACTER)
-		backtrack_byte (state, task);
+		backtrack_byte (state, subtask);
 	      else
-		put_byte (state->result, task);
+		put_byte (state->result, subtask);
 	      state = NULL;
 	    }
 	  else
 	    {
-	      put_byte (value, task);
-	      if (!get_ucs2 (&value, step, task))
+	      put_byte (value, subtask);
+	      if (!get_ucs2 (&value, subtask))
 		break;
 	    }
 	}
 
       if (state)
-	if (state->result == NOT_A_CHARACTER)
-	  backtrack_byte (state, task);
-	else
-	  put_byte (state->result, task);
+	{
+	  if (state->result == NOT_A_CHARACTER)
+	    backtrack_byte (state, subtask);
+	  else
+	    put_byte (state->result, subtask);
+	}
     }
 
-  TASK_RETURN (task);
+  SUBTASK_RETURN (subtask);
 }
 
 bool
-combine_byte_ucs2 (RECODE_CONST_STEP step, RECODE_TASK task)
+combine_byte_ucs2 (RECODE_SUBTASK subtask)
 {
   unsigned value;
 
-  if (value = get_byte (task), value != EOF)
+  if (value = get_byte (subtask), value != EOF)
     {
       struct state *state = NULL;
 
-      if (task->byte_order_mark)
-	put_ucs2 (BYTE_ORDER_MARK, task);
+      if (subtask->task->byte_order_mark)
+	put_ucs2 (BYTE_ORDER_MARK, subtask);
 
       while (true)
 	{
-	  struct state *shift = find_shifted_state (state, value, step);
+	  struct state *shift
+	    = find_shifted_state (state, value, subtask->step);
 
 	  if (shift)
 	    {
 	      state = shift;
-	      if (value = get_byte (task), value == EOF)
+	      if (value = get_byte (subtask), value == EOF)
 		break;
 	    }
 	  else if (state)
 	    {
 	      if (state->result == NOT_A_CHARACTER)
-		backtrack_ucs2 (state, task);
+		backtrack_ucs2 (state, subtask);
 	      else
-		put_ucs2 (state->result, task);
+		put_ucs2 (state->result, subtask);
 	      state = NULL;
 	    }
 	  else
 	    {
-	      put_ucs2 (value, task);
-	      if (value = get_byte (task), value == EOF)
+	      put_ucs2 (value, subtask);
+	      if (value = get_byte (subtask), value == EOF)
 		break;
 	    }
 	}
 
       if (state)
-	if (state->result == NOT_A_CHARACTER)
-	  backtrack_ucs2 (state, task);
-	else
-	  put_ucs2 (state->result, task);
+	{
+	  if (state->result == NOT_A_CHARACTER)
+	    backtrack_ucs2 (state, subtask);
+	  else
+	    put_ucs2 (state->result, subtask);
+	}
     }
 
-  TASK_RETURN (task);
+  SUBTASK_RETURN (subtask);
 }
 
 bool
-combine_ucs2_ucs2 (RECODE_CONST_STEP step, RECODE_TASK task)
+combine_ucs2_ucs2 (RECODE_SUBTASK subtask)
 {
   unsigned value;
 
-  if (get_ucs2 (&value, step, task))
+  if (get_ucs2 (&value, subtask))
     {
       struct state *state = NULL;
 
-      if (task->byte_order_mark)
-	put_ucs2 (BYTE_ORDER_MARK, task);
+      if (subtask->task->byte_order_mark)
+	put_ucs2 (BYTE_ORDER_MARK, subtask);
 
       while (true)
 	{
-	  struct state *shift = find_shifted_state (state, value, step);
+	  struct state *shift
+	    = find_shifted_state (state, value, subtask->step);
 
 	  if (shift)
 	    {
 	      state = shift;
-	      if (!get_ucs2 (&value, step, task))
+	      if (!get_ucs2 (&value, subtask))
 		break;
 	    }
 	  else if (state)
 	    {
 	      if (state->result == NOT_A_CHARACTER)
-		backtrack_ucs2 (state, task);
+		backtrack_ucs2 (state, subtask);
 	      else
-		put_ucs2 (state->result, task);
+		put_ucs2 (state->result, subtask);
 	      state = NULL;
 	    }
 	  else
 	    {
-	      put_ucs2 (value, task);
-	      if (!get_ucs2 (&value, step, task))
+	      put_ucs2 (value, subtask);
+	      if (!get_ucs2 (&value, subtask))
 		break;
 	    }
 	}
 
       if (state)
-	if (state->result == NOT_A_CHARACTER)
-	  backtrack_ucs2 (state, task);
-	else
-	  put_ucs2 (state->result, task);
+	{
+	  if (state->result == NOT_A_CHARACTER)
+	    backtrack_ucs2 (state, subtask);
+	  else
+	    put_ucs2 (state->result, subtask);
+	}
     }
 
-  TASK_RETURN (task);
+  SUBTASK_RETURN (subtask);
 }
