@@ -1,19 +1,19 @@
 /* Conversion of files between different charsets and surfaces.
-   Copyright  1997, 98, 99 Free Software Foundation, Inc.
+   Copyright  1997, 98, 99, 00 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
    Contributed by Franois Pinard <pinard@iro.umontreal.ca>, 1997.
 
-   The `recode' Library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public License
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.
 
-   The `recode' Library is distributed in the hope that it will be
+   This library is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with the `recode' Library; see the file `COPYING.LIB'.
    If not, write to the Free Software Foundation, Inc., 59 Temple Place -
    Suite 330, Boston, MA 02111-1307, USA.  */
@@ -21,33 +21,33 @@
 #include "common.h"
 
 static bool
-permute_21 (RECODE_CONST_STEP step, RECODE_TASK task)
+permute_21 (RECODE_SUBTASK subtask)
 {
   int character1;
   int character2;
 
   while (true)
     {
-      character1 = get_byte (task);
+      character1 = get_byte (subtask);
       if (character1 == EOF)
 	break;
 
-      character2 = get_byte (task);
+      character2 = get_byte (subtask);
       if (character2 == EOF)
 	{
-	  put_byte (character1, task);
+	  put_byte (character1, subtask);
 	  break;
 	}
 
-      put_byte (character2, task);
-      put_byte (character1, task);
+      put_byte (character2, subtask);
+      put_byte (character1, subtask);
     }
 
-  TASK_RETURN (task);
+  SUBTASK_RETURN (subtask);
 }
 
 static bool
-permute_4321 (RECODE_CONST_STEP step, RECODE_TASK task)
+permute_4321 (RECODE_SUBTASK subtask)
 {
   int character1;
   int character2;
@@ -56,41 +56,41 @@ permute_4321 (RECODE_CONST_STEP step, RECODE_TASK task)
 
   while (true)
     {
-      character1 = get_byte (task);
+      character1 = get_byte (subtask);
       if (character1 == EOF)
 	break;
 
-      character2 = get_byte (task);
+      character2 = get_byte (subtask);
       if (character2 == EOF)
 	{
-	  put_byte (character1, task);
+	  put_byte (character1, subtask);
 	  break;
 	}
 
-      character3 = get_byte (task);
+      character3 = get_byte (subtask);
       if (character3 == EOF)
 	{
-	  put_byte (character2, task);
-	  put_byte (character1, task);
+	  put_byte (character2, subtask);
+	  put_byte (character1, subtask);
 	  break;
 	}
 
-      character4 = get_byte (task);
+      character4 = get_byte (subtask);
       if (character4 == EOF)
 	{
-	  put_byte (character3, task);
-	  put_byte (character2, task);
-	  put_byte (character1, task);
+	  put_byte (character3, subtask);
+	  put_byte (character2, subtask);
+	  put_byte (character1, subtask);
 	  break;
 	}
 
-      put_byte (character4, task);
-      put_byte (character3, task);
-      put_byte (character2, task);
-      put_byte (character1, task);
+      put_byte (character4, subtask);
+      put_byte (character3, subtask);
+      put_byte (character2, subtask);
+      put_byte (character1, subtask);
     }
 
-  TASK_RETURN (task);
+  SUBTASK_RETURN (subtask);
 }
 
 bool
@@ -111,3 +111,8 @@ module_permutations (RECODE_OUTER outer)
 		       NULL, permute_4321)
     && declare_alias (outer, "swabytes", "21-Permutation");
 }
+
+void
+delmodule_permutations (RECODE_OUTER outer)
+{
+}
