@@ -1,19 +1,18 @@
 /* Conversion of files between different charsets and surfaces.
-   Copyright  1990, 92, 93, 94, 96, 97, 98, 99 Free Software Foundation, Inc.
-   This file is part of the GNU C Library.
+   Copyright  1990,92,93,94,96,97,98,99,00 Free Software Foundation, Inc.
    Contributed by Franois Pinard <pinard@iro.umontreal.ca>, 1990.
 
-   The `recode' Library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public License
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.
 
-   The `recode' Library is distributed in the hope that it will be
+   This library is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with the `recode' Library; see the file `COPYING.LIB'.
    If not, write to the Free Software Foundation, Inc., 59 Temple Place -
    Suite 330, Boston, MA 02111-1307, USA.  */
@@ -57,7 +56,7 @@ recode_error (RECODE_OUTER outer, const char *format, ...)
 #else
 recode_error (outer, format, va_alist)
      RECODE_OUTER outer;
-     char *format;
+     const char *format;
      va_dcl
 #endif
 {
@@ -84,10 +83,11 @@ recode_perror (RECODE_OUTER outer, const char *format, ...)
 #else
 recode_perror (outer, format, va_alist)
      RECODE_OUTER outer;
-     char *format;
+     const char *format;
      va_dcl
 #endif
 {
+  int saved_errno = errno;
 #ifdef VA_START
   va_list args;
 
@@ -101,7 +101,7 @@ recode_perror (outer, format, va_alist)
 #else
   fprintf (stderr, format, a1, a2, a3, a4, a5, a6, a7, a8);
 #endif
-  fprintf (stderr, ": %s\n", strerror (errno));
+  fprintf (stderr, ": %s\n", strerror (saved_errno));
   fflush (stderr);
 }
 
@@ -356,33 +356,32 @@ complete_pairs (RECODE_OUTER outer, RECODE_STEP step,
 `-------------------------------------------------------------------------*/
 
 bool
-transform_byte_to_ucs2 (RECODE_CONST_STEP step,
-			RECODE_TASK task)
+transform_byte_to_ucs2 (RECODE_SUBTASK subtask)
 {
   int input_char;		/* current character */
   int output_value;		/* value being output */
 
-  if (input_char = get_byte (task), input_char != EOF)
+  if (input_char = get_byte (subtask), input_char != EOF)
     {
-      if (task->byte_order_mark)
-	put_ucs2 (BYTE_ORDER_MARK, task);
+      if (subtask->task->byte_order_mark)
+	put_ucs2 (BYTE_ORDER_MARK, subtask);
 
       while (input_char != EOF)
 	{
-	  output_value = code_to_ucs2 (step->before, input_char);
+	  output_value = code_to_ucs2 (subtask->step->before, input_char);
 	  if (output_value < 0)
 	    {
-	      RETURN_IF_NOGO (RECODE_UNTRANSLATABLE, step, task);
-	      put_ucs2 (REPLACEMENT_CHARACTER, task);
+	      RETURN_IF_NOGO (RECODE_UNTRANSLATABLE, subtask);
+	      put_ucs2 (REPLACEMENT_CHARACTER, subtask);
 	    }
 	  else
-	    put_ucs2 (output_value, task);
+	    put_ucs2 (output_value, subtask);
 
-	  input_char = get_byte (task);
+	  input_char = get_byte (subtask);
 	}
     }
 
-  TASK_RETURN (task);
+  SUBTASK_RETURN (subtask);
 }
 
 /*-------------------------------------------------------------------------.
@@ -426,14 +425,21 @@ init_ucs2_to_byte (RECODE_STEP step,
     return false;
 
   if (!ALLOC (data, 256, struct ucs2_to_byte))
-    return false;
+    {
+      hash_free (table);
+      return false;
+    }
 
   for (counter = 0; counter < 256; counter++)
     {
       data[counter].code = code_to_ucs2 (step->after, counter);
       data[counter].byte = counter;
       if (!hash_insert (table, data + counter))
-	return false;
+	{
+	  hash_free (table);
+	  free (data);
+	  return false;
+	}
     }
 
   step->local = table;
@@ -441,25 +447,24 @@ init_ucs2_to_byte (RECODE_STEP step,
 }
 
 bool
-transform_ucs2_to_byte (RECODE_CONST_STEP step,
-			RECODE_TASK task)
+transform_ucs2_to_byte (RECODE_SUBTASK subtask)
 {
-  Hash_table *table = step->local;
+  Hash_table *table = subtask->step->local;
   struct ucs2_to_byte lookup;
   struct ucs2_to_byte *entry;
   unsigned input_value;		/* current UCS-2 character */
 
-  while (get_ucs2 (&input_value, step, task))
+  while (get_ucs2 (&input_value, subtask))
     {
       lookup.code = input_value;
       entry = hash_lookup (table, &lookup);
       if (entry)
-	put_byte (entry->byte, task);
+	put_byte (entry->byte, subtask);
       else
-	RETURN_IF_NOGO (RECODE_UNTRANSLATABLE, step, task);
+	RETURN_IF_NOGO (RECODE_UNTRANSLATABLE, subtask);
     }
 
-  TASK_RETURN (task);
+  SUBTASK_RETURN (subtask);
 }
 
 /* Table editing on stdout.  */
@@ -576,6 +581,9 @@ recode_format_table (RECODE_REQUEST request,
 
       switch (header_language)
 	{
+	case RECODE_NO_LANGUAGE:
+	  assert (0);
+
 	case RECODE_LANGUAGE_C:
 	  printf ("unsigned char const %s[256] =\n", name);
 	  printf ("  {\n");
@@ -595,6 +603,9 @@ recode_format_table (RECODE_REQUEST request,
 	}
       switch (header_language)
 	{
+	case RECODE_NO_LANGUAGE:
+	  assert (0);
+
 	case RECODE_LANGUAGE_C:
 	  printf ("  };\n");
 	  break;
@@ -612,6 +623,9 @@ recode_format_table (RECODE_REQUEST request,
 
       switch (header_language)
 	{
+	case RECODE_NO_LANGUAGE:
+	  assert (0);
+
 	case RECODE_LANGUAGE_C:
 	  printf ("const char *%s[256] =\n", name);
 	  printf ("  {\n");
@@ -689,6 +703,9 @@ recode_format_table (RECODE_REQUEST request,
 	  else
 	    switch (header_language)
 	      {
+	      case RECODE_NO_LANGUAGE:
+		assert (0);
+
 	      case RECODE_LANGUAGE_C:
 		printf ("0");
 		column++;
@@ -710,6 +727,9 @@ recode_format_table (RECODE_REQUEST request,
 	}
       switch (header_language)
 	{
+	case RECODE_NO_LANGUAGE:
+	  assert (0);
+
 	case RECODE_LANGUAGE_C:
 	  printf ("  };\n");
 	  break;
