@@ -1,25 +1,70 @@
 /* Conversion of files between different charsets and surfaces.
-   Copyright  1990, 92, 93, 94, 96, 97, 98, 99 Free Software Foundation, Inc.
-   This file is part of the GNU C Library.
+   Copyright  1990,92,93,94,96,97,98,99,00 Free Software Foundation, Inc.
    Contributed by Franois Pinard <pinard@iro.umontreal.ca>, 1990.
 
-   The `recode' Library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public License
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.
 
-   The `recode' Library is distributed in the hope that it will be
+   This library is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with the `recode' Library; see the file `COPYING.LIB'.
    If not, write to the Free Software Foundation, Inc., 59 Temple Place -
    Suite 330, Boston, MA 02111-1307, USA.  */
 
 #include "common.h"
 
+#if HAVE_UNISTD_H
+# include <unistd.h>
+#endif
+
+#include <sys/types.h>
+
+#include <sys/wait.h>
+/* Bruno suggests the following, from GNU make 3.79.0.2 in `job.c'.  He also
+   remarks that on Linux, WEXITSTATUS are bits 15..8 and WTERMSIG are bits
+   7..0, while BeOS uses the contrary.  */
+#if HAVE_UNION_WAIT
+# define WAIT_T union wait
+# ifndef WTERMSIG
+#  define WTERMSIG(x) ((x).w_termsig)
+# endif
+# ifndef WCOREDUMP
+#  define WCOREDUMP(x) ((x).w_coredump)
+# endif
+# ifndef WEXITSTATUS
+#  define WEXITSTATUS(x) ((x).w_retcode)
+# endif
+# ifndef WIFSIGNALED
+#  define WIFSIGNALED(x) (WTERMSIG(x) != 0)
+# endif
+# ifndef WIFEXITED
+#  define WIFEXITED(x) (WTERMSIG(x) == 0)
+# endif
+#else
+# define WAIT_T int
+# ifndef WTERMSIG
+#  define WTERMSIG(x) ((x) & 0x7f)
+# endif
+# ifndef WCOREDUMP
+#  define WCOREDUMP(x) ((x) & 0x80)
+# endif
+# ifndef WEXITSTATUS
+#  define WEXITSTATUS(x) (((x) >> 8) & 0xff)
+# endif
+# ifndef WIFSIGNALED
+#  define WIFSIGNALED(x) (WTERMSIG (x) != 0)
+# endif
+# ifndef WIFEXITED
+#  define WIFEXITED(x) (WTERMSIG (x) == 0)
+# endif
+#endif
+
 /* Buffer size used in transform_mere_copy.  */
 #define BUFFER_SIZE (16 * 1024)
 
@@ -59,27 +104,47 @@ get_byte_helper (RECODE_TASK task)
    generated code.  */
 
 void
-put_byte_helper (int byte, RECODE_TASK task)
+put_byte_helper (int byte, RECODE_SUBTASK subtask)
 {
-  if (task->output.file)
-    putc (byte, task->output.file);
-  else if (task->output.cursor == task->output.limit)
+  if (subtask->output.file)
+    putc (byte, subtask->output.file);
+  else if (subtask->output.cursor == subtask->output.limit)
     {
-      RECODE_OUTER outer = task->request->outer;
-      size_t old_size = task->output.limit - task->output.buffer;
+      RECODE_OUTER outer = subtask->task->request->outer;
+      size_t old_size = subtask->output.limit - subtask->output.buffer;
       size_t new_size = old_size * 3 / 2 + 40;
 
       /* FIXME: Rethink about how the error should be reported.  */
 
-      if (REALLOC (task->output.buffer, new_size, char))
+      if (REALLOC (subtask->output.buffer, new_size, char))
 	{
-	  task->output.cursor = task->output.buffer + old_size;
-	  task->output.limit = task->output.buffer + new_size;
-	  *task->output.cursor++ = byte;
+	  subtask->output.cursor = subtask->output.buffer + old_size;
+	  subtask->output.limit = subtask->output.buffer + new_size;
+	  *subtask->output.cursor++ = byte;
 	}
     }
   else
-    *task->output.cursor++ = byte;
+    *subtask->output.cursor++ = byte;
+}
+
+/* Error processing.  */
+
+/*------------------------------------------------------------------------.
+| Handle a given ERROR, while executing STEP within TASK.  Return true if |
+| the abort level has been reached.                                       |
+`------------------------------------------------------------------------*/
+
+bool
+recode_if_nogo (enum recode_error new_error, RECODE_SUBTASK subtask)
+{
+  RECODE_TASK task = subtask->task;
+
+  if (new_error > task->error_so_far)
+    {
+      task->error_so_far = new_error;
+      task->error_at_step = subtask->step;
+    }
+  return task->error_so_far >= task->abort_level;
 }
 
 /* Recoding execution control.  */
@@ -89,47 +154,47 @@ put_byte_helper (int byte, RECODE_TASK task)
 `--------------*/
 
 static bool
-transform_mere_copy (RECODE_TASK task)
+transform_mere_copy (RECODE_SUBTASK subtask)
 {
-  if (task->input.file && task->output.file)
+  if (subtask->input.file && subtask->output.file)
     {
       /* File to file.  */
 
       char buffer[BUFFER_SIZE];
       size_t size;
 
-      while (size = fread (buffer, 1, BUFFER_SIZE, task->input.file),
+      while (size = fread (buffer, 1, BUFFER_SIZE, subtask->input.file),
 	     size == BUFFER_SIZE)
-	if (fwrite (buffer, BUFFER_SIZE, 1, task->output.file) != 1)
+	if (fwrite (buffer, BUFFER_SIZE, 1, subtask->output.file) != 1)
 	  {
 	    recode_perror (NULL, "fwrite ()");
 	    return false;
 	  }
       if (size > 0)
-	if (fwrite (buffer, size, 1, task->output.file) != 1)
+	if (fwrite (buffer, size, 1, subtask->output.file) != 1)
 	  {
 	    recode_perror (NULL, "fwrite ()");
 	    return false;
 	  }
     }
-  else if (task->input.file)
+  else if (subtask->input.file)
     {
       /* File to buffer.  */
 
       int character;
 
       /* FIXME: buy now, pay (optimise) only later.   */
-      while (character = get_byte (task), character != EOF)
-	put_byte (character, task);
+      while (character = get_byte (subtask), character != EOF)
+	put_byte (character, subtask);
     }
-  else if (task->output.file)
+  else if (subtask->output.file)
     {
       /* Buffer to file.  */
 
-      if (task->input.cursor < task->input.limit)
-	if (fwrite (task->input.cursor,
-		    (unsigned) (task->input.limit - task->input.cursor),
-		    1, task->output.file)
+      if (subtask->input.cursor < subtask->input.limit)
+	if (fwrite (subtask->input.cursor,
+		    (unsigned) (subtask->input.limit - subtask->input.cursor),
+		    1, subtask->output.file)
 	    != 1)
 	  {
 	    recode_perror (NULL, "fwrite ()");
@@ -143,8 +208,8 @@ transform_mere_copy (RECODE_TASK task)
       int character;
 
       /* FIXME: buy now, pay (optimise) only later.  */
-      while (character = get_byte (task), character != EOF)
-	put_byte (character, task);
+      while (character = get_byte (subtask), character != EOF)
+	put_byte (character, subtask);
     }
 
   return true;
@@ -155,16 +220,15 @@ transform_mere_copy (RECODE_TASK task)
 `--------------------------------------------------*/
 
 bool
-transform_byte_to_byte (RECODE_CONST_STEP step,
-			RECODE_TASK task)
+transform_byte_to_byte (RECODE_SUBTASK subtask)
 {
-  unsigned const char *table = step->step_table;
+  unsigned const char *table = subtask->step->step_table;
   int input_char;
 
-  while (input_char = get_byte (task), input_char != EOF)
-    put_byte (table[input_char], task);
+  while (input_char = get_byte (subtask), input_char != EOF)
+    put_byte (table[input_char], subtask);
 
-  TASK_RETURN (task);
+  SUBTASK_RETURN (subtask);
 }
 
 /*---------------------------------------------------.
@@ -172,26 +236,25 @@ transform_byte_to_byte (RECODE_CONST_STEP step,
 `---------------------------------------------------*/
 
 bool
-transform_byte_to_variable (RECODE_CONST_STEP step,
-			    RECODE_TASK task)
+transform_byte_to_variable (RECODE_SUBTASK subtask)
 {
-  const char *const *table = step->step_table;
+  const char *const *table = subtask->step->step_table;
   int input_char;
   const char *output_string;
 
   /* Copy the file through the one to many recoding table.  */
 
-  while (input_char = get_byte (task), input_char != EOF)
+  while (input_char = get_byte (subtask), input_char != EOF)
     if (output_string = table[input_char], output_string)
       while (*output_string)
 	{
-	  put_byte (*output_string, task);
+	  put_byte (*output_string, subtask);
 	  output_string++;
 	}
    else
-     RETURN_IF_NOGO (RECODE_UNTRANSLATABLE, step, task);
+     RETURN_IF_NOGO (RECODE_UNTRANSLATABLE, subtask);
 
-  TASK_RETURN (task);
+  SUBTASK_RETURN (subtask);
 }
 
 /*---------------------------------------------------------------------.
@@ -204,24 +267,23 @@ static bool
 perform_memory_sequence (RECODE_TASK task)
 {
   RECODE_CONST_REQUEST request = task->request;
-  RECODE_TASK subtask = recode_new_task (request);
+  struct recode_subtask subtask_block;
+  RECODE_SUBTASK subtask = &subtask_block;
   struct recode_read_write_text input;
   struct recode_read_write_text output;
   unsigned sequence_index;
   RECODE_CONST_STEP step;
 
-  if (!subtask)
-    return false;
-
-  subtask->byte_order_mark = task->byte_order_mark;
+  memset (subtask, 0, sizeof (struct recode_subtask));
   memset (&input, 0, sizeof (struct recode_read_write_text));
   memset (&output, 0, sizeof (struct recode_read_write_text));
+  subtask->task = task;
 
   /* Execute one pass for each step of the sequence.  */
 
   for (sequence_index = 0;
        sequence_index < request->sequence_length
-	 && subtask->error_so_far < subtask->abort_level;
+	 && task->error_so_far < task->abort_level;
        sequence_index++)
     {
       /* Select the input text for this step.  */
@@ -231,14 +293,16 @@ perform_memory_sequence (RECODE_TASK task)
 	  subtask->input = task->input;
 
 	  if (subtask->input.name)
-	    if (!*subtask->input.name)
-	      subtask->input.file = stdin;
-	    else if (subtask->input.file = fopen (subtask->input.name, "r"),
-		     subtask->input.file == NULL)
-	      {
-		recode_perror (NULL, "fopen (%s)", subtask->input.name);
-		return false;
-	      }
+	    {
+	      if (!*subtask->input.name)
+		subtask->input.file = stdin;
+	      else if (subtask->input.file = fopen (subtask->input.name, "r"),
+		       subtask->input.file == NULL)
+		{
+		  recode_perror (NULL, "fopen (%s)", subtask->input.name);
+		  return false;
+		}
+	    }
 	}
       else
 	{
@@ -259,25 +323,24 @@ perform_memory_sequence (RECODE_TASK task)
 	  subtask->output = task->output;
 
 	  if (subtask->output.name)
-	    if (!*subtask->output.name)
-	      subtask->output.file = stdout;
-	    else if (subtask->output.file = fopen (subtask->output.name, "w"),
-		     subtask->output.file == NULL)
-	      {
-		recode_perror (NULL, "fopen (%s)", subtask->output.name);
-		return false;
-	      }
+	    {
+	      if (!*subtask->output.name)
+		subtask->output.file = stdout;
+	      else if (subtask->output.file = fopen (subtask->output.name,
+						     "w"),
+		       subtask->output.file == NULL)
+		{
+		  recode_perror (NULL, "fopen (%s)", subtask->output.name);
+		  return false;
+		}
+	    }
 	}
 
       /* Execute one recoding step.  */
 
       step = request->sequence_array + sequence_index;
-      (*step->transform_routine) (step, subtask);
-      if (subtask->error_so_far > task->error_so_far)
-	{
-	  task->error_so_far = subtask->error_so_far;
-	  task->error_at_step = subtask->error_at_step;
-	}
+      subtask->step = step;
+      (*step->transform_routine) (subtask);
 
       /* Post-step clean up.  */
 
@@ -294,7 +357,7 @@ perform_memory_sequence (RECODE_TASK task)
 
       /* Prepare for next step.  */
 
-      subtask->swap_input = RECODE_SWAP_UNDECIDED;
+      task->swap_input = RECODE_SWAP_UNDECIDED;
 
       if (sequence_index < request->sequence_length - 1)
 	{
@@ -320,8 +383,7 @@ perform_memory_sequence (RECODE_TASK task)
   if (output.buffer)
     free (output.buffer);
 
-  /* How does subtask get freed?  FIXME!  */
-  TASK_RETURN (subtask);
+  SUBTASK_RETURN (subtask);
 }
 
 /*-------------------------------------------------------------------------.
@@ -387,7 +449,8 @@ static bool
 perform_pass_sequence (RECODE_TASK task)
 {
   RECODE_CONST_REQUEST request = task->request;
-  RECODE_TASK subtask = recode_new_task (request);
+  struct recode_subtask subtask_block;
+  RECODE_SUBTASK subtask = &subtask_block;
   struct recode_read_write_text input;
   struct recode_read_write_text output;
   unsigned sequence_index;
@@ -397,12 +460,10 @@ perform_pass_sequence (RECODE_TASK task)
   char temporary_name_2[L_tmpnam];
 #endif
 
-  if (!subtask)
-    return false;
-
-  subtask->byte_order_mark = task->byte_order_mark;
+  memset (subtask, 0, sizeof (struct recode_subtask));
   memset (&input, 0, sizeof (struct recode_read_write_text));
   memset (&output, 0, sizeof (struct recode_read_write_text));
+  subtask->task = task;
 
 #if USE_TMPNAM
 # if DOSWIN_OR_OS2
@@ -421,7 +482,7 @@ perform_pass_sequence (RECODE_TASK task)
 
   for (sequence_index = 0;
        sequence_index < request->sequence_length
-	 && subtask->error_so_far < subtask->abort_level;
+	 && task->error_so_far < task->abort_level;
        sequence_index++)
     {
       /* Select the input text for this step.  */
@@ -431,14 +492,16 @@ perform_pass_sequence (RECODE_TASK task)
 	  subtask->input = task->input;
 
 	  if (subtask->input.name)
-	    if (!*subtask->input.name)
-	      subtask->input.file = stdin;
-	    else if (subtask->input.file = fopen (subtask->input.name, "r"),
-		     subtask->input.file == NULL)
-	      {
-		recode_perror (NULL, "fopen (%s)", subtask->input.name);
-		return false;
-	      }
+	    {
+	      if (!*subtask->input.name)
+		subtask->input.file = stdin;
+	      else if (subtask->input.file = fopen (subtask->input.name, "r"),
+		       subtask->input.file == NULL)
+		{
+		  recode_perror (NULL, "fopen (%s)", subtask->input.name);
+		  return false;
+		}
+	    }
 	}
       else
 	{
@@ -489,25 +552,24 @@ perform_pass_sequence (RECODE_TASK task)
 	  subtask->output = task->output;
 
 	  if (subtask->output.name)
-	    if (!*subtask->output.name)
-	      subtask->output.file = stdout;
-	    else if (subtask->output.file = fopen (subtask->output.name, "w"),
-		     subtask->output.file == NULL)
-	      {
-		recode_perror (NULL, "fopen (%s)", subtask->output.name);
-		return false;
-	      }
+	    {
+	      if (!*subtask->output.name)
+		subtask->output.file = stdout;
+	      else if (subtask->output.file = fopen (subtask->output.name,
+						     "w"),
+		       subtask->output.file == NULL)
+		{
+		  recode_perror (NULL, "fopen (%s)", subtask->output.name);
+		  return false;
+		}
+	    }
 	}
 
       /* Execute one recoding step.  */
 
       step = request->sequence_array + sequence_index;
-      (*step->transform_routine) (step, subtask);
-      if (subtask->error_so_far > task->error_so_far)
-	{
-	  task->error_so_far = subtask->error_so_far;
-	  task->error_at_step = subtask->error_at_step;
-	}
+      subtask->step = step;
+      (*step->transform_routine) (subtask);
 
       /* Post-step clean up.  */
 
@@ -537,7 +599,7 @@ perform_pass_sequence (RECODE_TASK task)
 
       /* Prepare for next step.  */
 
-      subtask->swap_input = RECODE_SWAP_UNDECIDED;
+      task->swap_input = RECODE_SWAP_UNDECIDED;
 
       if (sequence_index < request->sequence_length - 1)
 	{
@@ -552,8 +614,8 @@ perform_pass_sequence (RECODE_TASK task)
 	  task->output = subtask->output;
 	}
     }
-  /* How does subtask get freed?  FIXME!  */
-  TASK_RETURN (subtask);
+
+  SUBTASK_RETURN (subtask);
 }
 
 #if HAVE_PIPE
@@ -630,22 +692,29 @@ perform_pipe_sequence (RECODE_TASK task)
 {
   RECODE_CONST_REQUEST request = task->request;
   RECODE_OUTER outer = request->outer;
+  struct recode_subtask subtask_block;
+  RECODE_SUBTASK subtask = &subtask_block;
 
   unsigned sequence_index;	/* index into sequence */
-  RECODE_CONST_STEP step; /* pointer into single_steps */
+  RECODE_CONST_STEP step;	/* pointer into single_steps */
 
   int pipe_pair[2];		/* pair of file descriptors for a pipe */
   int child_process;		/* child process number, zero if child */
-  int wait_status;		/* status returned by wait() */
+  WAIT_T wait_status;		/* status returned by wait() */
+
+  memset (subtask, 0, sizeof (struct recode_subtask));
+  subtask->task = task;
+  subtask->input = task->input;
+  subtask->output = task->output;
 
   /* Prepare the final output file.  */
 
-  if (!*task->output.name)
-    task->output.file = stdout;
-  else if (task->output.file = fopen (task->output.name, "w"),
-	   task->output.file == NULL)
+  if (!*subtask->output.name)
+    subtask->output.file = stdout;
+  else if (subtask->output.file = fopen (subtask->output.name, "w"),
+	   subtask->output.file == NULL)
     {
-      recode_perror (outer, "fopen (%s)", task->output.name);
+      recode_perror (outer, "fopen (%s)", subtask->output.name);
       return false;
     }
 
@@ -676,20 +745,21 @@ perform_pipe_sequence (RECODE_TASK task)
 	      recode_perror (outer, "close ()");
 	      return false;
 	    }
-	  if (task->input.file = fdopen (pipe_pair[0], "r"),
-	      task->input.file == NULL)
+	  if (subtask->input.file = fdopen (pipe_pair[0], "r"),
+	      subtask->input.file == NULL)
 	    {
 	      recode_perror (outer, "fdopen ()");
 	      return false;
 	    }
 
 	  step = request->sequence_array + sequence_index;
-	  (*step->transform_routine) (step, task);
+	  subtask->step = step;
+	  (*step->transform_routine) (subtask);
 
-	  fclose (task->input.file);
+	  fclose (subtask->input.file);
 	  if (sequence_index < request->sequence_length - 1
-	      || *task->output.name)
-	    fclose (task->output.file);
+	      || *subtask->output.name)
+	    fclose (subtask->output.file);
 
 	  exit (task->error_so_far < task->fail_level ? EXIT_SUCCESS
 		: EXIT_FAILURE);
@@ -698,7 +768,7 @@ perform_pipe_sequence (RECODE_TASK task)
 	{
           /* The parent redirects the current output file to the pipe.  */
 
-	  if (dup2 (pipe_pair[1], fileno (task->output.file)) < 0)
+	  if (dup2 (pipe_pair[1], fileno (subtask->output.file)) < 0)
 	    {
 	      recode_perror (outer, "dup2 ()");
 	      return false;
@@ -719,32 +789,37 @@ perform_pipe_sequence (RECODE_TASK task)
   /* All the children are created, blocked on read.  Now, feed the whole
      chain of processes with the output of the first recoding step.  */
 
-  if (!*task->input.name)
-    task->input.file = stdin;
-  else if (task->input.file = fopen (task->input.name, "r"),
-	   task->input.file == NULL)
+  if (!*subtask->input.name)
+    subtask->input.file = stdin;
+  else if (subtask->input.file = fopen (subtask->input.name, "r"),
+	   subtask->input.file == NULL)
     {
-      recode_perror (outer, "fopen (%s)", task->input.name);
+      recode_perror (outer, "fopen (%s)", subtask->input.name);
       return false;
     }
 
   step = request->sequence_array;
-  (*step->transform_routine) (step, task);
+  subtask->step = step;
+  (*step->transform_routine) (subtask);
 
-  if (*task->input.name)
-    fclose (task->input.file);
+  if (*subtask->input.name)
+    fclose (subtask->input.file);
 
-  fclose (task->output.file);
+  fclose (subtask->output.file);
 
   /* Wait on all children, mainly to avoid synchronisation problems on
      output file contents, but also to reduce the number of zombie
      processes in case the user recodes many files at once.  */
 
-  /* FIXME: Maybe I should rather use <wait.h> and the macros?  */
   while (wait (&wait_status) > 0)
     {
       /* Diagnose and abort on any abnormally terminating child.  */
 
+#if 0				/* FIXME: Bruno would prefer this.  */
+      if (!(WIFEXITED (wait_status)
+	    || (WIFSIGNALED (wait_status)
+		&& WTERMSIG (wait_status) == SIGPIPE)))
+#endif
       if ((wait_status & MASK (8)) != 0
 	  && (wait_status & MASK (8)) != SIGPIPE)
 	{
@@ -755,7 +830,9 @@ perform_pipe_sequence (RECODE_TASK task)
 
       /* Check for a nonzero exit from the terminating child.  */
 
-      if (wait_status & MASK (16))
+      if (WIFEXITED (wait_status)
+	  ? WEXITSTATUS (wait_status) != 0
+	  : WTERMSIG (wait_status) != 0)
 	/* FIXME: It is not very clear what happened in sub-processes.  */
 	if (task->error_so_far < task->fail_level)
 	  {
@@ -774,7 +851,7 @@ perform_pipe_sequence (RECODE_TASK task)
       }
 #endif
 
-  TASK_RETURN (task);
+  SUBTASK_RETURN (subtask);
 }
 
 #else /* not 1 */
@@ -783,31 +860,38 @@ static bool
 perform_pipe_sequence (RECODE_TASK task)
 {
   RECODE_CONST_REQUEST request = task->request;
+  struct recode_subtask subtask_block;
+  RECODE_SUBTASK subtask = &subtask_block;
 
   unsigned sequence_index;	/* index into sequence */
-  RECODE_CONST_STEP step; /* pointer into single_steps */
+  RECODE_CONST_STEP step;	/* pointer into single_steps */
 
   int pipe_pair[2];		/* pair of file descriptors for a pipe */
   int child_process;		/* child process number, zero if child */
-  int wait_status;		/* status returned by wait() */
+  WAIT_T wait_status;		/* status returned by wait() */
+
+  memset (subtask, 0, sizeof (struct recode_subtask));
+  subtask->task = task;
+  subtask->input = task->input;
+  subtask->output = task->output;
 
   /* Prepare the final files.  */
 
-  if (!*task->input.name)
-    task->input.file = stdin;
-  else if (task->input.file = fopen (task->input.name, "r"),
-	   task->input.file == NULL)
+  if (!*subtask->input.name)
+    subtask->input.file = stdin;
+  else if (subtask->input.file = fopen (subtask->input.name, "r"),
+	   subtask->input.file == NULL)
     {
-      recode_perror (outer, "fopen (%s)", task->input.name);
+      recode_perror (outer, "fopen (%s)", subtask->input.name);
       return false;
     }
 
-  if (!*task->output.name)
-    task->output.file = stdout;
-  else if (task->output.file = fopen (task->output.name, "w"),
-	   task->output.file == NULL)
+  if (!*subtask->output.name)
+    subtask->output.file = stdout;
+  else if (subtask->output.file = fopen (subtask->output.name, "w"),
+	   subtask->output.file == NULL)
     {
-      recode_perror (outer, "fopen (%s)", task->output.name);
+      recode_perror (outer, "fopen (%s)", subtask->output.name);
       return false;
     }
 
@@ -838,20 +922,21 @@ perform_pipe_sequence (RECODE_TASK task)
 	      recode_perror (outer, "close ()");
 	      return false;
 	    }
-	  if (task->input.file = fdopen (pipe_pair[0], "r"),
-	      task->input.file == NULL)
+	  if (subtask->input.file = fdopen (pipe_pair[0], "r"),
+	      subtask->input.file == NULL)
 	    {
 	      recode_perror (outer, "fdopen ()");
 	      return false;
 	    }
 
 	  step = request->sequence_array[sequence_index];
-	  (*step->transform_routine) (step, task);
+	  subtask->step = step;
+	  (*step->transform_routine) (subtask);
 
-	  fclose (task->input.file);
+	  fclose (subtask->input.file);
 	  if (sequence_index < request->sequence_length - 1
-	      || *task->output.name)
-	    fclose (task->output.file);
+	      || *subtask->output.name)
+	    fclose (subtask->output.file);
 
 	  exit (task->error_so_far < task->fail_level ? EXIT_SUCCESS
 		: EXIT_FAILURE);
@@ -860,7 +945,7 @@ perform_pipe_sequence (RECODE_TASK task)
 	{
           /* The parent redirects the current output file to the pipe.  */
 
-	  if (dup2 (pipe_pair[1], fileno (task->output.file)) < 0)
+	  if (dup2 (pipe_pair[1], fileno (subtask->output.file)) < 0)
 	    {
 	      recode_perror (outer, "dup2 ()");
 	      return false;
@@ -884,22 +969,23 @@ perform_pipe_sequence (RECODE_TASK task)
   /* All the children are created, blocked on read.  Now, feed the whole
      chain of processes with the output of the first recoding step.  */
 
-  if (!*task->input.name)
-    task->input.file = stdin;
-  else if (task->input.file = fopen (task->input.name, "r"),
-	   task->input.file == NULL)
+  if (!*subtask->input.name)
+    subtask->input.file = stdin;
+  else if (subtask->input.file = fopen (subtask->input.name, "r"),
+	   subtask->input.file == NULL)
     {
-      recode_perror (outer, "fopen (%s)", task->input.name);
+      recode_perror (outer, "fopen (%s)", subtask->input.name);
       return false;
     }
 
   step = request->sequence_array[0];
-  (*step->transform_routine) (step, task);
+  subtask->step = step;
+  (*step->transform_routine) (subtask);
 
-  if (*task->input.name)
-    fclose (task->input.file);
+  if (*subtask->input.name)
+    fclose (subtask->input.file);
 
-  fclose (task->output.file);
+  fclose (subtask->output.file);
 
   /* Wait on all children, mainly to avoid synchronisation problems on
      output file contents, but also to reduce the number of zombie
@@ -909,8 +995,9 @@ perform_pipe_sequence (RECODE_TASK task)
     {
       /* Diagnose and abort on any abnormally terminating child.  */
 
-      if ((wait_status & MASK (8)) != 0
-	  && (wait_status & MASK (8)) != SIGPIPE)
+      if (!(WIFEXITED (wait_status)
+	    || (WIFSIGNALED (wait_status)
+		&& WTERMSIG (wait_status) == SIGPIPE)))
 	{
 	  recode_error (outer, _("Child process wait status is 0x%0.2x"),
 			wait_status);
@@ -919,7 +1006,9 @@ perform_pipe_sequence (RECODE_TASK task)
 
       /* Check for a nonzero exit from the terminating child.  */
 
-      if (wait_status & MASK (16))
+      if (WIFEXITED (wait_status)
+	  ? WEXITSTATUS (wait_status) != 0
+	  : WTERMSIG (wait_status) != 0)
 	/* FIXME: It is not very clear what happened in sub-processes.  */
 	if (task->error_so_far < task->fail_level)
 	  {
@@ -936,7 +1025,7 @@ perform_pipe_sequence (RECODE_TASK task)
 	task->error_at_step = step;
       }
 
-  TASK_RETURN (task);
+  SUBTASK_RETURN (subtask);
 }
 
 #endif /* not 1 */
@@ -959,7 +1048,7 @@ recode_new_task (RECODE_CONST_REQUEST request)
   RECODE_TASK task;
 
   if (!ALLOC (task, 1, struct recode_task))
-    return false;
+    return NULL;
 
   memset (task, 0, sizeof (struct recode_task));
   task->request = request;
@@ -1003,8 +1092,6 @@ bool
 recode_perform_task (RECODE_TASK task)
 {
   RECODE_CONST_REQUEST request = task->request;
-
-  RECODE_CONST_STEP step;
   bool success;
 
 #if DOSWIN_OR_OS2
@@ -1067,40 +1154,54 @@ recode_perform_task (RECODE_TASK task)
       }
   else
     {
+      struct recode_subtask subtask_block;
+      RECODE_SUBTASK subtask = &subtask_block;
+
       /* Execute a simple recoding (a single step, or no step at all).  */
 
-      if (task->input.name)
-	if (!*task->input.name)
-	  task->input.file = stdin;
-	else if (task->input.file = fopen (task->input.name, "r"),
-		 task->input.file == NULL)
-	  {
-	    recode_perror (NULL, "fopen (%s)", task->input.name);
-	    return false;
-	  }
+      memset (subtask, 0, sizeof (struct recode_subtask));
+      subtask->task = task;
+      subtask->input = task->input;
+      subtask->output = task->output;
 
-      if (task->output.name)
-	if (!*task->output.name)
-	  task->output.file = stdout;
-	else if (task->output.file = fopen (task->output.name, "w"),
-		 task->output.file == NULL)
-	  {
-	    recode_perror (NULL, "fopen (%s)", task->output.name);
-	    return false;
-	  }
+      if (subtask->input.name)
+	{
+	  if (!*subtask->input.name)
+	    subtask->input.file = stdin;
+	  else if (subtask->input.file = fopen (subtask->input.name, "r"),
+		   subtask->input.file == NULL)
+	    {
+	      recode_perror (NULL, "fopen (%s)", subtask->input.name);
+	      return false;
+	    }
+	}
+
+      if (subtask->output.name)
+	{
+	  if (!*subtask->output.name)
+	    subtask->output.file = stdout;
+	  else if (subtask->output.file = fopen (subtask->output.name, "w"),
+		   subtask->output.file == NULL)
+	    {
+	      recode_perror (NULL, "fopen (%s)", subtask->output.name);
+	      return false;
+	    }
+	}
 
       if (request->sequence_length == 1)
 	{
-	  step = request->sequence_array;
-	  success = (*step->transform_routine) (step, task);
+	  RECODE_CONST_STEP step = request->sequence_array;
+
+	  subtask->step = step;
+	  success = (*step->transform_routine) (subtask);
 	}
       else
-	success = transform_mere_copy (task);
+	success = transform_mere_copy (subtask);
 
-      if (task->input.name && *task->input.name)
-	fclose (task->input.file);
-      if (task->output.name && *task->output.name)
-	fclose (task->output.file);
+      if (subtask->input.name && *subtask->input.name)
+	fclose (subtask->input.file);
+      if (subtask->output.name && *subtask->output.name)
+	fclose (subtask->output.file);
     }
 
   return success;
